import { system, world, ItemStack } from "@minecraft/server";

const type = {
  default: "express:carrier",
  chest: "express:chest_carrier",
  barrel: "express:barrel_carrier",
  spawner: "express:spawner_carrier",
};

world.beforeEvents.playerInteractWithBlock.subscribe((ev) => {
  const { player, block, itemStack } = ev;
  const itemId = itemStack?.typeId;
  const { x, y, z, location, dimension } = block;
  const inv = player.getComponent("inventory").container;

  const structureTypes = {
    "minecraft:chest": type.chest,
    "minecraft:barrel": type.barrel,
    "minecraft:mob_spawner": type.spawner,
  };

  const blockType = block.typeId;

  if (itemId === type.default && structureTypes[blockType]) {
    ev.cancel = true;
    const random = Math.floor(Math.random() * 10000000000);
    const itemType = structureTypes[blockType];
    const item = new ItemStack(itemType);

    player.runCommandAsync(
      `structure save ${random}_storage ${x} ${y} ${z} ${x} ${y} ${z} false disk true`
    );

    system.run(() => {
      item.setLore([`§r§7Id: ${random}`]);
      inv.setItem(player.selectedSlot, item);
      dimension.fillBlocks(location, location, "minecraft:air");
    });
  }

  if (
    [type.chest, type.barrel, type.spawner].includes(itemId) &&
    blockType === "express:block_support"
  ) {
    system.run(() => {
      const carrier = new ItemStack(type.default);
      inv.setItem(player.selectedSlot, carrier);
      const lore = itemStack.getLore().toString().replace("§r§7Id: ", "");
      player.runCommandAsync(`structure load ${lore}_storage ${x} ${y} ${z}`);
      player.runCommandAsync(`structure delete ${lore}_storage`);
    });
  }
});
